import pickle, datetime, cgi, cgitb, sys
cgitb.enable()

print "Content-Type: text/html\n"


# Grab the username
form = cgi.FieldStorage()

if form.has_key("newusername") and form["newusername"].value.isalnum():
    username = form["newusername"].value
elif form.has_key("username") and form["username"].value.isalnum():
    username = form["username"].value
else:
    print """\
<html>
<head>
<title>Drawing Night Scheduler</title>
<link rel="stylesheet" type="text/css" href="style.css"/>
</head>
<body>
<h1>Error: Invalid input.</h1>
Enter the timetable via <a href="dnscheduler.py">here</a>.
</body>
</html>
"""
    sys.exit()

# Grab the timezone
try:
    timezone = int(form["timezone"].value)
except:
    timezone = 0
tzshift = datetime.timedelta(0, timezone*3600)

# Load the preexisting data.
# data = {username:[datetimestr, ...], ...}
try:
    data = pickle.load(open("data"))
except:
    data = {}
data[username] = data.get(username, [])

# Set up the time constants.
accuratenow = datetime.datetime.utcnow()+tzshift
now = datetime.datetime(*accuratenow.timetuple()[:3])
weekdays = ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"]
shiftday = datetime.timedelta(1)
shifthour = datetime.timedelta(0, 3600)


# Conversion from datetime to string format.
def datetime2str(dt):
    return "%04i/%02i/%02i/%02i" % dt.timetuple()[:4]

# Record who belongs to what date.
date2users = {}
for user in data:
    if user != username:
        for dtstr in data[user]:
            date2users[dtstr] = date2users.get(dtstr, [])+[user+"<br/>"]

# Find the max number of users on one date
maxusers = 0 if len(date2users)==0 else max([len(i) for i in date2users.values()])
idealdates = [i for i in date2users if len(date2users[i])==maxusers]

# Start HTML output
print """\
<html>

<head>
<title>Drawing Night Scheduler</title>
<link rel="stylesheet" type="text/css" href="style.css"/>
<script language="javascript">
"""

# Give existing data to the Javascript.
print """\
filledcells = %s;
idealcells = %s;
selectedcells = %s;
username = %s;
""" % (date2users.keys(), idealdates, data[username], repr(username))

# Output Javascript code and more HTML.
print """\
function init () {
    for (i in filledcells) {
        document.getElementById(filledcells[i]).style.backgroundColor = "green";
    }
    for (i in idealcells) {
        document.getElementById(idealcells[i]).style.backgroundColor = "lime";
    }
    for (i in selectedcells) {
        document.getElementById(selectedcells[i]).style.backgroundColor = "blue";
    }
}

function current2utc (utcstr) {
    d = new Date();
    ymdh = utcstr.split("/");
    d.setUTCFullYear(ymdh[0], ymdh[1]-1, ymdh[2]);
    d.setUTCHours(ymdh[3]);
    d.setUTCMinutes(0);
    d.setUTCSeconds(0);
    return d.toUTCString().concat("<br/><br/>");
}

function updateOutput (handle) {
    outtext = current2utc(handle.id);
    outtext = outtext.concat("<span class='userlist'>");
    outtext = outtext.concat(handle.getElementsByTagName("div")[0].innerHTML);
    if (handle.getElementsByTagName("input")[0].checked) {outtext = outtext.concat(username);}
    outtext = outtext.concat("</span>");
    document.getElementById("output").innerHTML = outtext;
}

function toggledCell (handle) {
    //alert(5);
    if (handle.checked) {
        handle.parentNode.style.backgroundColor = "blue";
    } else {
        handle.parentNode.style.backgroundColor = "white";
        id = handle.parentNode.id;
        for (i in filledcells) {
            if (filledcells[i]==id) {document.getElementById(id).style.backgroundColor = "green";}
        }
        for (i in idealcells) {
            if (idealcells[i]==id) {document.getElementById(id).style.backgroundColor = "lime";}
        }
    }
    updateOutput(handle.parentNode);
}
</script>
</head>

<body onload="init()">

<h1>Drawing Night Scheduler</h1>

<div class="instructions">
Time slots that have people in them are highlighted <span style="color:green">green</span>, the ones that have the most people are <span style="color:lime">bright green</span>.<br/>
Click on time slots to toggle your availability, these will be highlighted <span style="color:blue">blue</span>.<br/>
Beside the table is information about the cell your mouse is over; the time converted to UTC and who has selected that time slot.
</div>

<form id="form" method="post" action="submit.py">
<input type="hidden" name="username" value="%s"/>
<input type="hidden" name="timezone" value="%s"/>
Times are displayed in your timezone: UTC%+i<br/>
At time of page load it was %s in your timezone.
<table class="timetable">
""" % (username, timezone, timezone, accuratenow)

# Generate the HTML for the cells.
#<tr>
#    <th>00</th>
#    <td id="2009/08/31/00" onmouseover="updateOutput(this)">
#        <input type="checkbox" name="2009/08/31/00" onchange="toggledCell(this)"/>
#        <div>Aooueao<br/>Rrkkn<br/>Bpgbbpcgj<br/></div>
#    </td>
#    <td id="2009/09/01/00" onmouseover="updateOutput(this)">
#        <input type="checkbox" name="2009/09/01/00" onchange="toggledCell(this)" checked="on"/>
#        <div></div>
#    </td>
#</tr>

# Top row
print "<tr><th></th>",
for day in range(14):
    d = now + (shiftday*day)
    print "<th>%s %s</th>" % (weekdays[d.weekday()], d.day),
print "</tr>"
# Main cells
for hour in range(24):
    d1 = now + (shifthour*hour)
    print "<tr><th>%02i:00</th>" % d1.hour,
    for day in range(14):
        d2 = d1 + (shiftday*day)
        dtstr = datetime2str(d2-tzshift)
        print """<td id="%s" onmouseover="updateOutput(this)">""" % dtstr,
        print """<input type="checkbox" name="%s" onchange="toggledCell(this)" %s>""" % (dtstr, 'checked="on"' if dtstr in data[username] else ""),
        print """<div>%s</div>""" % ("".join(date2users.get(dtstr, []))),
        print "</td>",
    print "</tr>"




print """\
</table>
<div id="output"></div>
<input type="submit" value="Save"/>
</form>

<a href="dnscheduler.py">Change user</a>
</body>


</html>
"""

